<?php
namespace Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\GER\Utilities\Environment;
use Zeedhi\Framework\DTO\Response\Error;

class Fai33000LancamentoDeDespesasDiversas extends \Teknisa\Libs\Controller\CRUD {
	
	protected $dataSourceName;
	protected $parameterBagColumns = array('CDFILIAL', 'DATAFINAL', 'DATAINICIAL', 'ALLDATA', 'CDDESPPREV');
	protected $dataSourceNameToSave;
	protected $connection;
	protected $dataSourceManager;

	public function __construct($crudService, $parameterBag, $dataSourceManager, $connection){
		$this->dataSourceManager = $dataSourceManager;
		$this->crudService 		 = $crudService;
		$this->parameterBag 	 = $parameterBag;
		$this->connection		 = $connection;
	}
	
	public function atualizaDespesa(DTO\Request $request, DTO\Response $response){
       
        $dataSet = $request->getDataSet("DataSet");
        
        if($dataSet != null){
        	$rows = $dataSet->getRows();
        	
        	foreach ($rows as $row) {
        	
	            $impostoBackupRow = $row[0];
	            $impostoCurrentRow  = $row[1];
	            
	            
	            try{
	                $stmt = $this->connection->prepare("UPDATE DESPPREVFILI
	                 SET
	                   DTPRDESPFILI = :P_DATA_LANCAMENTO,
	                   VRPRDESPFILI = :P_VALOR_PREVISTO,
	                   VRREDESPFILI = :P_VALOR_REAL
	                 WHERE
	                   CDFILIAL   = :P_CDFILIAL   AND
	                   CDDESPPREV  = :P_CDDESPPREV  AND
	                   DTPRDESPFILI = :P_DTPRDESPFILI");
	    
	                $stmt->execute(
	                    array(
	                        'P_DATA_LANCAMENTO' => $impostoCurrentRow["DTPRDESPFILI"],
	                        'P_VALOR_PREVISTO' => $impostoCurrentRow["VRPRDESPFILI"],
	                        'P_VALOR_REAL' => $impostoCurrentRow["VRREDESPFILI"],
	                        'P_CDFILIAL' => $impostoBackupRow["CDFILIAL"],
	                        'p_CDDESPPREV' => $impostoBackupRow["CDDESPPREV"],
	                        'P_DTPRDESPFILI' => $impostoBackupRow["DTPRDESPFILI"]
	                    )
	                );
	            }catch(\Exception $ex){
	                if(strpos($ex->getMessage(), "ORA-00001") === FALSE){
	                    $response->setError(new Error("ERRO_OPERACAO2" . $ex->getMessage(), 500));
	                }else{
	                    $response->setError(new Error("Não é possível alterar. Despesa já cadastrada para essa data!", 500));
	                }
	            }
        	}
        }
    }

	public function autoOperation(DTO\Request $request, DTO\Response $response){
		
		try{
			$row = $request->getRow();
			$despesasCadastradas = $this->findDespprev($row);
			$dataSave = $this->mountDataSave($row, $despesasCadastradas);
			$this->saveDespprev($dataSave);
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

	private function saveDespprev(&$dataSave){
		
		$currentTime = $this->crudService->getEnvironment()->getCurrentTime()->format("d/m/Y H:i:s");
		foreach($dataSave as &$row){
			$conditions = $this->mountConditions($row);
			$dataDespprev = $this->crudService->getFilteredDataSet('despprevfili', $conditions)->getRows();
			if(!empty($dataDespprev)){
				$dataDespprev = $dataDespprev[0];
				$row['VRPRDESPFILI'] = $row['VRPRDESPFILI'] + $dataDespprev['VRPRDESPFILI'];
				$row['VRREDESPFILI'] = $row['VRREDESPFILI'] + $dataDespprev['VRREDESPFILI'];
				$row['IDATIVO'] = $dataDespprev['IDATIVO'];
				$row['DTINCLUSAO'] = $dataDespprev['DTINCLUSAO'];
				$row['NRORGINCLUSAO'] = $dataDespprev['NRORGINCLUSAO'];
				$row['CDOPERINCLUSAO'] = $dataDespprev['CDOPERINCLUSAO'];
				$row['__is_new'] = false;
			} else {
				$row['DTINCLUSAO'] = $currentTime;
				$row['NRORGINCLUSAO'] = $this->crudService->getEnvironment()->getNrOrg();
				$row['CDOPERINCLUSAO'] = $this->crudService->getEnvironment()->getCdOperador();
				$row['IDATIVO'] = 'S';
			}
			$row['DTULTATU'] = $currentTime;
			$row['NRORGULTATU'] = $this->crudService->getEnvironment()->getNrOrg();
			$row['CDOPERULTATU'] = $this->crudService->getEnvironment()->getCdOperador();
		}
		$this->crudService->save('despprevfili',$dataSave);
	}

	private function mountConditions($row){
		
		$conditions = array(
			array(
				'columnName' => 'CDFILIAL',
				'operator' => '=',
				'value' => $row['CDFILIAL']
			),
			array(
				'columnName' => 'CDDESPPREV',
				'operator' => '=',
				'value' => $row['CDDESPPREV']
			),
			array(
				'columnName' => 'DTPRDESPFILI',
				'operator' => '=',
				'value' => $row['DTPRDESPFILI']
			),
			array(
				'columnName' => 'NRORG',
				'operator' => '=',
				'value' => $row['NRORG']
			)
		);

		return $conditions;
	}

	private function findDespprev($row){
		$query = "SELECT
				    DESPPREVFILI.*,
				    FILIAL.NMFILIAL,
				    DESPPREV.NMDESPPREV
				FROM
				    DESPPREVFILI
				    JOIN FILIAL ON FILIAL.CDFILIAL = DESPPREVFILI.CDFILIAL
				    JOIN DESPPREV ON DESPPREVFILI.CDDESPPREV = DESPPREV.CDDESPPREV
				                     AND DESPPREVFILI.NRORG = DESPPREV.NRORG
				WHERE
				    ( DESPPREVFILI.CDFILIAL =:CDFILIAL )
				    AND INSTR(:CDDESPPREV, DESPPREVFILI.CDDESPPREV) <> 0 
				    AND DTPRDESPFILI BETWEEN :DATAINICIAL AND :DATAFINAL
				ORDER BY
				    DESPPREVFILI.CDDESPPREV";
		$params = array(
			"CDFILIAL" => $row["CDFILIAL"],
			"DATAINICIAL" => $row["DATAINICIAL"],
			"DATAFINAL" => $row["DATAFINAL"],
			"CDDESPPREV" => $row["CDDESPPREV"]
		);
		$stmt = $this->connection->prepare($query);
		$stmt->execute($params);
		$dataSet = $stmt->fetchAll();
		return $dataSet;
		// $conditions = array(
		// 	array(
		// 		"columnName" => "CDFILIAL",
		// 		"operator" => "=",
		// 		"value" => $this->crudService->getEnvironment()->getCdFilial()
		// 	),
		// 	array(
		// 		"columnName" => "CDDESPPREV",
		// 		"operator" => "IN",
		// 		"value" => $row['CDDESPPREV']
		// 	),
		// 	array(
		// 		"columnName" => "DTPRDESPFILI",
		// 		"operator" => "BETWEEN",
		// 		"value" => array($row['DATAINICIAL'], $row["DATAFINAL"])
		// 	),
		// 	array(
		// 		"columnName" => "NRORG",
		// 		"operator" => "=",
		// 		"value" => $this->crudService->getEnvironment()->getNrOrg()
		// 	)
		// );
		// $filterCriteria = new FilterCriteria('fai33000_despprevfili_despprev', $conditions);
		// $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
		// $dataSet = $this->dataSourceManager->findBy($newFilterCriteria)->getRows();
		// var_dump($dataSet);die;
	
	}

	private function mountDataSave($row, $despesasCadastradas){
		
		$dataSave = array();

		if($row['OPERATION'] == 2) {
			$function = 'mountReleaseDate';
		} else {
			$function = 'mountFirstDayDate';
		}
		foreach($despesasCadastradas as $despesa){
			
			$novaDataLancamento = call_user_func_array(array($this, $function), array($row['DTLANCAMENTO'], $despesa['DTPRDESPFILI']));
			$novaDespesa = array(
				'CDDESPPREV' => $despesa['CDDESPPREV'],
				'CDFILIAL' => $despesa['CDFILIAL'],
				'CDPRODUTO' => $despesa['CDPRODUTO'],
				'DTPRDESPFILI' => $novaDataLancamento,
				'NRORG' => $despesa['NRORG'],
				'VRPRDESPFILI' => $despesa['VRPRDESPFILI'],
				'VRREDESPFILI' => $despesa['VRREDESPFILI'],
				'__is_new' => true
			);
			$this->comparaDataSet($dataSave, $novaDespesa);
		}
		return $dataSave;
	}

	private function comparaDataSet(&$dataSave, $newRow){
		$change = false;
		
		foreach($dataSave as &$row){
			if(($row['CDDESPPREV'] == $newRow['CDDESPPREV']) && ($row['CDFILIAL'] == $newRow['CDFILIAL']) &&  ($row['CDPRODUTO'] == $newRow['CDPRODUTO']) && ($row['DTPRDESPFILI'] == $newRow['DTPRDESPFILI'])){
				$row['VRPRDESPFILI'] = $row['VRPRDESPFILI'] + $newRow['VRPRDESPFILI'];
				$row['VRREDESPFILI'] = $row['VRREDESPFILI'] + $newRow['VRREDESPFILI'];
				return true;
			}
		}
		array_push($dataSave, $newRow);
	}

	private function mountReleaseDate($dtLancamento, $oldDate){
		
		$day =  substr($oldDate, 0, 2);
		$lastDayOfMonth = substr($dtLancamento, 0, 2);
		$newDate;
		if(intval($day) > intval($lastDayOfMonth)){
			$newDate = $dtLancamento;
		} else {
			$newDate = $day.substr($dtLancamento, 2, 10);
		}
		return $newDate;
	}

	private function mountFirstDayDate($dtLnacamento, $oldDate){
		return $dtLnacamento;
	}

}